@@ -1,4 +1,6 @@
 import subprocess
+import threading
+import time
 
 import click
 from rq import Queue
@@ -54,10 +56,21 @@ def develop():
     help="Skip launching Flask, so that Flask can be managed externally",
 )
 def debug(port, skip_flask):
+    from dallinger.command_line.utils import verify_package
+
+    if not verify_package():
+        # We could instead use the @require_exp_directory decorator,
+        # but this doesn't print anything useful without the verbose flag.
+        # To consider for later: improving this default behavior of @require_exp_directory?
+        print(
+            "Cannot continue, there is a problem with the current experiment (see above)."
+        )
+        raise click.Abort
+
     _bootstrap()
 
     q = Queue("default", connection=redis_conn)
-    job = q.enqueue_call(launch_app_and_open_dashboard, kwargs={"port": port})
+    job = q.enqueue_call(launch_app_and_open_browser, kwargs={"port": port})
 
     if not skip_flask:
         config = get_config()
@@ -82,9 +95,11 @@ def _bootstrap(exp_config=None):
     bootstrapper.run()
 
 
-def launch_app_and_open_dashboard(port):
+def launch_app_and_open_browser(port):
     _launch_app(port)
-    _open_dashboard(port)
+    _async_browser("dashboard", port)
+    time.sleep(0.1)  # A little delay to ensure they always open in the same order
+    _async_browser("ad", port)
 
 
 def _launch_app(port):
@@ -92,10 +107,6 @@ def _launch_app(port):
     handle_launch_data(url, error=log, delay=1.0, context="local")
 
 
-def _open_dashboard(port):
-    _browser("dashboard", port)
-
-
 @develop.command()
 @click.option(
     "--route",
@@ -109,14 +120,21 @@ def browser(route=None, port=5000):
 
 def _browser(route=None, port=5000):
     """Open one of the supported routes with appropriate path and URL parameters"""
-    config = get_config()
-    config.load()
+    config = get_config(load=True)
     url_factory = valid_routes.get(route)
     if url_factory is not None:
         url = url_factory(config, port)
-        log(f"Experiment dashboard: {url}")
+        log(f"Opening {route}: {url}")
         open_browser(url)
     else:
         click.echo(
             "Supported routes are:\n\t{}".format("\n\t".join(valid_routes.keys()))
         )
+
+
+def _async_browser(route=None, port=5000):
+    # The _browser call is pretty slow, so we run it in a thread
+    # to allow us to open multiple browsers at the same time
+    threading.Thread(
+        target=_browser, name=f"Open {route}", kwargs={"route": route, "port": port}
+    ).start()
