@@ -1,5 +1,6 @@
 import inspect
 import os
+import platform
 import re
 import sys
 import tempfile
@@ -8,6 +9,10 @@ from functools import wraps
 import click
 
 from dallinger.config import get_config, initialize_experiment_package
+from dallinger.constraints import (
+    _get_requested_python_version,
+    _python_versions_consistent,
+)
 from dallinger.utils import ExperimentFileSource
 from dallinger.version import __version__
 
@@ -81,6 +86,7 @@ def verify_package(verbose=True):
     """
     results = (
         verify_directory(verbose),
+        verify_python_version(verbose),
         verify_experiment_module(verbose),
         verify_config(verbose),
         verify_no_conflicts(verbose),
@@ -91,7 +97,7 @@ def verify_package(verbose=True):
     return ok
 
 
-def verify_directory(verbose=True, max_size_mb=50):
+def verify_directory(verbose=True):
     """Ensure that the current directory looks like a Dallinger experiment, and
     does not appear to have unintended contents that will be copied on
     deployment.
@@ -108,22 +114,27 @@ def verify_directory(verbose=True, max_size_mb=50):
             log("✗ {} is MISSING".format(f), chevrons=False, verbose=verbose)
             ok = False
 
+    # Get experiment max size in MB from environment variable, default to 256MB
+    exp_max_size_mb = int(os.environ.get("EXP_MAX_SIZE_MB", "256"))
+
     # Check size
-    max_size = max_size_mb * mb_to_bytes
+    max_size = exp_max_size_mb * mb_to_bytes
     file_source = ExperimentFileSource(os.getcwd())
     size = file_source.size
     size_in_mb = round(size / mb_to_bytes)
     if size <= max_size:
         log(
-            "✓ Size OK at {}MB (max is {}MB)".format(size_in_mb, max_size_mb),
+            "✓ Size OK at {}MB (max is {}MB)".format(size_in_mb, exp_max_size_mb),
             chevrons=False,
             verbose=verbose,
         )
     else:
         log(
-            "✗ {}MB is TOO BIG (greater than {}MB)\n\tIncluded files:\n\t{}".format(
-                size_in_mb, max_size_mb, "\n\t".join(file_source.files)
-            ),
+            (
+                "✗ {}MB is TOO BIG (greater than {}MB). "
+                "You can override this limit by setting the EXP_MAX_SIZE_MB environment variable.\n"
+                "\tIncluded files:\n\t{}"
+            ).format(size_in_mb, exp_max_size_mb, "\n\t".join(file_source.files)),
             chevrons=False,
             verbose=verbose,
         )
@@ -132,6 +143,32 @@ def verify_directory(verbose=True, max_size_mb=50):
     return ok
 
 
+def verify_python_version(verbose):
+    """Verify that the current Python version is consistent with the .python-version file (if provided).
+    Note that this test is skipped if SKIP_PYTHON_VERSION_CHECK is set (which is the case in our CI tests).
+    """
+    # We don't force users to provide a .python-version file, but if they do, it must be respected.
+    if not os.path.exists(".python-version"):
+        return True
+    if os.environ.get("SKIP_PYTHON_VERSION_CHECK"):
+        return True
+    requested_python_version = _get_requested_python_version()
+    actual_python_version = platform.python_version()
+    if not _python_versions_consistent(requested_python_version, actual_python_version):
+        log(
+            (
+                f"✗ Running Python version {actual_python_version} which is not consistent "
+                f"with the version stated in .python-version ({requested_python_version}). "
+                f"Please either update the contents of your .python-version file to {actual_python_version} "
+                f"or install Python version {requested_python_version} into your local environment."
+            ),
+            chevrons=False,
+            verbose=verbose,
+        )
+        return False
+    return True
+
+
 def verify_experiment_module(verbose):
     """Perform basic sanity checks on experiment.py."""
     ok = True
