@@ -7,10 +7,12 @@ import sys
 import time
 from contextlib import contextmanager
 from functools import wraps
+from typing import Union
 
 import psycopg2
 from psycopg2.extensions import TransactionRollbackError
-from sqlalchemy import create_engine, event
+from rq import Queue
+from sqlalchemy import Table, create_engine, event
 from sqlalchemy.exc import OperationalError
 from sqlalchemy.ext.compiler import compiles
 from sqlalchemy.ext.declarative import declarative_base
@@ -20,7 +22,7 @@ from sqlalchemy.schema import DropTable
 from dallinger.config import initialize_experiment_package
 from dallinger.redis_utils import connect_to_redis
 
-logger = logging.getLogger("dallinger.db")
+logger = logging.getLogger(__name__)
 
 
 def corrected_db_url(db_url):
@@ -66,6 +68,19 @@ Consult the developer guide for more information.
 """
 
 
+def get_queue(name="default"):
+    """Return an rq.Queue with a connection to redis.
+
+    Optional param "name" should be one of:
+        - "high"
+        - "default"
+        - "low"
+
+    as these are the names of the queues workers read from.
+    """
+    return Queue(name, connection=redis_conn)
+
+
 def check_connection(timeout_secs=3):
     """Test that postgres is running and that we can connect using the
     configured URI.
@@ -84,12 +99,12 @@ def check_connection(timeout_secs=3):
 
 
 @contextmanager
-def sessions_scope(local_session, commit=False):
+def sessions_scope(session_=session, commit=False):
     """Provide a transactional scope around a series of operations."""
     try:
-        yield local_session
+        yield session_
         if commit:
-            local_session.commit()
+            session_.commit()
             logger.debug("DB session auto-committed as requested")
     except Exception as e:
         # We log the exception before re-raising it, in case the rollback also
@@ -98,10 +113,10 @@ def sessions_scope(local_session, commit=False):
         # This rollback is potentially redundant with the remove call below,
         # depending on how the scoped session is configured, but we'll be
         # explicit here.
-        local_session.rollback()
+        session_.rollback()
         raise e
     finally:
-        local_session.remove()
+        session_.remove()
         logger.debug("Session complete, db session closed")
 
 
@@ -110,7 +125,7 @@ def scoped_session_decorator(func):
 
     @wraps(func)
     def wrapper(*args, **kwargs):
-        with sessions_scope(session):
+        with sessions_scope():
             # The session used in func comes from the funcs globals, but
             # it will be a proxied thread local var from the session
             # registry, and will therefore be identical to the one returned
@@ -154,6 +169,106 @@ def init_db(drop_all=False, bind=engine):
     return session
 
 
+def get_all_mapped_classes():
+    """
+    Lists the different classes that are mapped with SQLAlchemy.
+    Classes are only included if they have at least one row in the database.
+    Returns a dictionary, keyed by class names,
+    where each value is itself a dictionary with three values:
+    ``cls``, the class itself;
+    ``table``, the database table within which the class can be found,
+    and ``polymorphic_identity``, the string label with which the class is
+    identified in the table's ``type`` column. The ``polymorphic_identity`` field
+    takes a value of ``None`` if the table does not use polymorphic inheritance.
+    """
+    classes = {}
+    for table in Base.metadata.tables.values():
+        if "type" in table.columns:
+            # Most Dallinger tables (e.g. Node, Network) have a type column that specifies which class
+            # is associated with that database row.
+            observed_types = [
+                r.type for r in session.query(table.columns.type).distinct().all()
+            ]
+            mapping = get_polymorphic_mapping(table)
+            for type_ in observed_types:
+                cls = mapping[type_]
+                classes[cls.__name__] = {
+                    "cls": cls,
+                    "table": table.name,
+                    "polymorphic_identity": type_,
+                }
+        else:
+            # Some tables (e.g. Notification) don't have any such column, so we can assume
+            # that they have exactly one mapped class.
+            if session.query(table.columns.id).count() > 0:
+                cls = get_mapped_class(table)
+                classes[cls.__name__] = {
+                    "cls": cls,
+                    "table": table.name,
+                    "polymorphic_identity": None,
+                }
+    return classes
+
+
+def get_mappers(table: Union[str, Table]):
+    if isinstance(table, str):
+        table_name = table
+    else:
+        assert isinstance(table, Table)
+        table_name = table.name
+
+    return [
+        mapper
+        for mapper in Base.registry.mappers
+        if table_name in [table.name for table in mapper.tables]
+    ]
+
+
+def get_polymorphic_mapping(table: Union[str, Table]):
+    """
+    Gets the polymorphic mapping for a given table.
+    Returns a dictionary
+    where the dictionary keys correspond to polymorphic identities
+    (i.e. possible values of the table's ``type`` column)
+    and the dictionary values correspond to classes.
+    """
+    mapping = {}
+    for mapper in get_mappers(table):
+        name = mapper.polymorphic_identity
+
+        # If we encounter two mappers with the same name, prioritize the one that is defined
+        # in the loaded Dallinger experiment
+        if name in mapping and mapping[name].__module__ == "dallinger_experiment":
+            continue
+
+        mapping[name] = mapper.class_
+    return mapping
+
+
+def get_mapped_classes(table: Union[str, Table]):
+    """
+    Returns a list of classes that map to the provided table.
+    """
+    return [
+        mapper.class_
+        for mapper in Base.registry.mappers
+        if mapper in get_mappers(table)
+    ]
+
+
+def get_mapped_class(table: Union[str, Table]):
+    """
+    Returns the single class that maps to the provided table.
+    Throws an ``AssertionError`` if there is not exactly one such class.
+    This function is therefore only intended for tables that do not implement
+    polymorphic identities, i.e. they do not include a ``type`` column.
+    An example is the Notification table.
+    """
+    mappers = get_mapped_classes(table)
+    assert len(mappers) == 1
+    return mappers[0]
+
+
 def serialized(func):
     """Run a function within a db transaction using SERIALIZABLE isolation.
 
