@@ -1,13 +1,13 @@
-import atexit
-import subprocess
-
-import gevent.monkey
-import werkzeug
-
-gevent.monkey.patch_all()  # Patch before importing app and all its dependencies
+import gevent.monkey  # noqa: E402
 
+gevent.monkey.patch_all()
+# ^ patch must happen at top of file before other imports
+import atexit  # noqa: E402
 import codecs  # noqa: E402
 import os  # noqa: E402
+import subprocess  # noqa: E402
+
+import werkzeug  # noqa: E402
 
 from dallinger.experiment_server.experiment_server import app  # noqa: E402, F401
 
