@@ -21,9 +21,11 @@ from typing import List, Optional, Union
 
 import requests
 from cached_property import cached_property
-from flask import Blueprint
-from sqlalchemy import Table, and_, create_engine, func
+from flask import Blueprint, url_for
+from markupsafe import escape
+from sqlalchemy import String, Table, and_, asc, cast, create_engine, desc, func, or_
 from sqlalchemy.orm import scoped_session, sessionmaker, undefer
+from sqlalchemy.orm.attributes import InstrumentedAttribute
 from sqlalchemy.orm.exc import MultipleResultsFound, NoResultFound
 
 from dallinger import db, models, recruiters
@@ -43,6 +45,7 @@ from dallinger.db import (
     get_polymorphic_mapping,
     init_db,
 )
+from dallinger.experiment_server.utils import date_handler
 from dallinger.heroku.tools import HerokuApp
 from dallinger.information import Gene, Meme, State
 from dallinger.models import Info, Network, Node, Participant, Transformation
@@ -76,6 +79,7 @@ def exp_class_working_dir(meth):
 class Experiment(object):
     """Define the structure of an experiment."""
 
+    _session = None
     app_id = None
     exp_config = None
     replay_path = "/"
@@ -110,6 +114,59 @@ class Experiment(object):
         static_folder="static",
     )
 
+    @classmethod
+    def get_index_html(cls):
+        return (
+            "<html><head></head><body><h1>Dallinger Experiment in progress</h1>"
+            "<p><a href={}>Dashboard</a></p></body></html>".format(
+                url_for("dashboard.dashboard_index")
+            )
+        )
+
+    #: Boolean, determines whether the experiment logs output when
+    #: running. Default is True.
+    verbose = True
+
+    #: String, the name of the experiment. Default is "Experiment
+    #: title".
+    task = "Experiment title"
+
+    #: int, the number of practice networks (see
+    #: :attr:`~dallinger.models.Network.role`). Default is 0.
+    practice_repeats = 0
+
+    #: int, the number of non practice networks (see
+    #: :attr:`~dallinger.models.Network.rle`). Default is 0.
+    experiment_repeats = 0
+
+    #: int, the number of participants
+    #: required to move from the waiting room to the experiment.
+    #: Default is 0 (no waiting room).
+    quorum = 0
+
+    #: int, the number of participants
+    #: requested when the experiment first starts. Default is 1.
+    initial_recruitment_size = 1
+
+    #: dictionary, the classes Dallinger can make in response
+    #: to front-end requests. Experiments can add new classes to this
+    #: dictionary.
+    known_classes = {
+        "Agent": Agent,
+        "Compression": Compression,
+        "Environment": Environment,
+        "Gene": Gene,
+        "Info": Info,
+        "Meme": Meme,
+        "Mutation": Mutation,
+        "Node": Node,
+        "Replication": Replication,
+        "Response": Response,
+        "Source": Source,
+        "State": State,
+        "Transformation": Transformation,
+    }
+
     #: Sequence of dashboard route/function names that should be excluded from
     #: rendering as tabs in the dashboard view.
     @classproperty
@@ -121,55 +178,12 @@ class Experiment(object):
         """A hook for custom organization of dashboard tabs in subclasses."""
         return tabs
 
-    def __init__(self, session=None):
-        """Create the experiment class. Sets the default value of attributes."""
+    def __init__(self, session=None, no_configure=False):
+        """Create the experiment class. Sets the default value of attributes.
 
-        #: Boolean, determines whether the experiment logs output when
-        #: running. Default is True.
-        self.verbose = True
-
-        #: String, the name of the experiment. Default is "Experiment
-        #: title".
-        self.task = "Experiment title"
-
-        #: session, the experiment's connection to the database.
-        self.session = session
-
-        #: int, the number of practice networks (see
-        #: :attr:`~dallinger.models.Network.role`). Default is 0.
-        self.practice_repeats = 0
-
-        #: int, the number of non practice networks (see
-        #: :attr:`~dallinger.models.Network.role`). Default is 0.
-        self.experiment_repeats = 0
-
-        #: int, the number of participants
-        #: required to move from the waiting room to the experiment.
-        #: Default is 0 (no waiting room).
-        self.quorum = 0
-
-        #: int, the number of participants
-        #: requested when the experiment first starts. Default is 1.
-        self.initial_recruitment_size = 1
-
-        #: dictionary, the classes Dallinger can make in response
-        #: to front-end requests. Experiments can add new classes to this
-        #: dictionary.
-        self.known_classes = {
-            "Agent": Agent,
-            "Compression": Compression,
-            "Environment": Environment,
-            "Gene": Gene,
-            "Info": Info,
-            "Meme": Meme,
-            "Mutation": Mutation,
-            "Node": Node,
-            "Replication": Replication,
-            "Response": Response,
-            "Source": Source,
-            "State": State,
-            "Transformation": Transformation,
-        }
+        :param session:  DEPRECATED. Database session is now managed automatically.
+            This parameter will be removed in Dallinger v13.x
+        """
 
         #: dictionary, the properties of this experiment that are exposed
         #: to the public over an AJAX call
@@ -178,6 +192,10 @@ class Experiment(object):
             self.public_properties = {}
 
         if session:
+            # BBB: Will trigger deprecation warning
+            self.session = session
+
+        if not no_configure:
             self.configure()
 
         try:
@@ -194,6 +212,41 @@ class Experiment(object):
         else:
             self.widget = module.ExperimentWidget(self)
 
+    @property
+    def session(self):
+        """DEPRECATED: Database session property.
+
+        This property is deprecated and will be removed in a future version. Use
+        'dallinger.db.session' instead. Alternatively, use
+        'dallinger.db.session_scope()' or
+        'dallinger.db.scoped_session_decorator' for explicit session management.
+
+        :returns: The database session (defaults to dallinger.db.session if not
+            explicitly set)
+        """
+        warnings.warn(
+            "The 'session' property is deprecated and will be removed in a future version. "
+            "Use dallinger.db.session instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        if self._session is not None:
+            return self._session
+        return db.session
+
+    @session.setter
+    def session(self, value):
+        """DEPRECATED: Set the database session.
+
+        This setter is deprecated and will be removed in Dallinger version 13.
+        """
+        warnings.warn(
+            "Setting the 'session' property is deprecated and will be removed in Dallinger version 13.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        self._session = value
+
     @staticmethod
     def before_request():
         return None
@@ -273,8 +326,8 @@ class Experiment(object):
 
     @property
     def background_tasks(self):
-        """An experiment may define functions or methods to be started as
-        background tasks upon experiment launch.
+        """Optional list of functions that will be run at launch time in
+        separate threads.
         """
         return []
 
@@ -344,9 +397,10 @@ class Experiment(object):
         """Async implementation of websocket message processing. Attempts to
         extract a participant id or node id from the message, and send the
         message to be processed asynchronously by
-        :func:`~dallinger.experiment.Experiment.receive_message` If it fails to
-        find a participant or node id in the message, then the message is
-        processed synchronously using
+        :func:`~dallinger.experiment.Experiment.receive_message` using the
+        Dallinger `worker_function`. If the message pyaload is JSON and contains
+        a property named `immediate` then the message will be processed
+        synchronously by
         :func:`~dallinger.experiment.Experiment.receive_message`.
 
         ``raw_message`` is a string that includes a channel name prefix, for
@@ -363,7 +417,7 @@ class Experiment(object):
         the experiment class that it uses for message responses then it's best
         to override this method instead of
         :func:`~dallinger.experiment.Experiment.receive_message`, and explicitly
-        hand off state synchronization and other database writes to async worker
+        hand off any state synchronization and database writes to async worker
         events.
 
         :param raw_message: a formatted message string ``'$channel_name:$data'``
@@ -376,26 +430,23 @@ class Experiment(object):
         try:
             message = json.loads(message_string)
         except Exception:
-            # Not JSON we have no information about the participant/node and
-            # will run synchonously
-            with db.sessions_scope():
-                self.receive_message(
-                    message_string, channel_name=channel_name, receive_time=receive_time
-                )
-            return
-
-        participant_id = (
-            message.get("sender")
-            or message.get("participant_id")
-            or message.get("client", {}).get("participant_id")
-        )
-        node_id = message.get("node_id")
-        if not participant_id and not node_id:
-            with db.sessions_scope():
-                self.receive_message(
-                    message_string, channel_name=channel_name, receive_time=receive_time
-                )
-            return
+            # We don't have JSON and can't determine participant or node id
+            participant_id = node_id = None
+        else:
+            participant_id = (
+                message.get("sender")
+                or message.get("participant_id")
+                or message.get("client", {}).get("participant_id")
+            )
+            node_id = message.get("node_id")
+            if message.get("immediate"):
+                with db.sessions_scope():
+                    self.receive_message(
+                        message_string,
+                        channel_name=channel_name,
+                        receive_time=receive_time,
+                    )
+                    return
 
         q = db.get_queue("high")
         q.enqueue(
@@ -427,13 +478,18 @@ class Experiment(object):
     def receive_message(
         self, message, channel_name=None, participant=None, node=None, receive_time=None
     ):
-        """Stub implementation of a websocket message processor. Messages
-        are are queued to be processed asynchronously by
-        :func:`~dallinger.experiment.Experiment.send` and the worker runs this
-        method to process those messages. Sub-classes that wish to handle
-        incoming messages asynchronously should override this method. Generally
-        this method should always be overridden whenever the ``Experiment``
-        :attr:`~dallinger.experiment.Experiment.channel` attribute is set.
+        """Stub implementation of a websocket message processor. The
+        :func:`~dallinger.experiment.Experiment.send` method either queues
+        incoming messages to be processed asynchronously using this method or
+        directly calls this method for messages flagged for `immediate`
+        processing.
+
+        Experiment classes that wish to handle incoming WebSocket messages
+        asynchronously should implement this method. An Experiment needs to
+        implement either this method or customize
+        :func:`~dallinger.experiment.Experiment.send` whenever the
+        ``Experiment`` :attr:`~dallinger.experiment.Experiment.channel`
+        attribute is set.
 
         ``message`` is a string, e.g. containing JSON formatted data.
 
@@ -441,16 +497,20 @@ class Experiment(object):
         connect/disconnect events are sent over the ``"dallinger_control"``
         channel.
 
-        This method is called synchronously when no participant or node
-        id can be determined from the message.
+        This method will be called synchronously in the
+        experiment web server by :func:`~dallinger.experiment.Experiment.send`
+        when the JSON message payload has an `immediate` property set. Otherwise,
+        it will be called asynchronously by a Dallinger worker process.
 
-        :param message: a websocket message
+        :param message: a websocket message string, usually JSON data
         :type message: str
 
-        :param channel_name: The name of the channel the message was received on.
+        :param channel_name: The name of the channel the message was received
+            on.
         :type channel_name: str
 
-        :param participant: the experiment participant object responsible for the message
+        :param participant: the experiment participant object responsible for
+            the message
         :type participant: :attr:`~dallinger.models.Participant` instance
 
         :param node: the experiment node the message corresponds to
@@ -494,18 +554,15 @@ class Experiment(object):
         # explicitly elsewhere, especially since the default constructor doesn't
         # call it.
         if not self.networks():
-            # This should generally be called from a flask route, but we use a
-            # session contextmanager to be safe.
-            with db.sessions_scope() as session:
-                for _ in range(self.practice_repeats):
-                    network = self.create_network()
-                    network.role = "practice"
-                    session.add(network)
-                for _ in range(self.experiment_repeats):
-                    network = self.create_network()
-                    network.role = "experiment"
-                    session.add(network)
-                session.commit()
+            for _ in range(self.practice_repeats):
+                network = self.create_network()
+                network.role = "practice"
+                db.session.add(network)
+            for _ in range(self.experiment_repeats):
+                network = self.create_network()
+                network.role = "experiment"
+                db.session.add(network)
+            db.session.commit()
 
     def create_network(self):
         """Return a new network."""
@@ -520,15 +577,15 @@ class Experiment(object):
 
         if full == "all":
             if role == "all":
-                return self.session.query(Network).all()
+                return db.session.query(Network).all()
             else:
-                return self.session.query(Network).filter_by(role=role).all()
+                return db.session.query(Network).filter_by(role=role).all()
         else:
             if role == "all":
-                return self.session.query(Network).filter_by(full=full).all()
+                return db.session.query(Network).filter_by(full=full).all()
             else:
                 return (
-                    self.session.query(Network)
+                    db.session.query(Network)
                     .filter(and_(Network.role == role, Network.full == full))
                     .all()
                 )
@@ -544,11 +601,11 @@ class Experiment(object):
         """
         key = participant.id
         networks_with_space = (
-            self.session.query(Network).filter_by(full=False).order_by(Network.id).all()
+            db.session.query(Network).filter_by(full=False).order_by(Network.id).all()
         )
         networks_participated_in = [
             node.network_id
-            for node in self.session.query(Node.network_id)
+            for node in db.session.query(Node.network_id)
             .filter_by(participant_id=participant.id)
             .all()
         ]
@@ -669,7 +726,7 @@ class Experiment(object):
             fingerprint_hash=fingerprint_hash,
             entry_information=entry_information,
         )
-        self.session.add(participant)
+        db.session.add(participant)
         return participant
 
     def load_participant(self, assignment_id):
@@ -684,7 +741,7 @@ class Experiment(object):
         """
         try:
             return (
-                self.session.query(Participant)
+                db.session.query(Participant)
                 .filter_by(assignment_id=assignment_id)
                 .one()
             )
@@ -895,7 +952,7 @@ class Experiment(object):
 
     def log_summary(self):
         """Log a summary of all the participants' status codes."""
-        participants = self.session.query(Participant.status).all()
+        participants = db.session.query(Participant.status).all()
         counts = Counter([p.status for p in participants])
         sorted_counts = sorted(counts.items(), key=itemgetter(0))
         self.log("Status summary: {}".format(str(sorted_counts)))
@@ -907,8 +964,8 @@ class Experiment(object):
         This only needs to be done for networks and participants.
         """
         if len(objects) > 0:
-            self.session.add_all(objects)
-        self.session.commit()
+            db.session.add_all(objects)
+        db.session.commit()
 
     def node_post_request(self, participant, node):
         """Run when a request to make a node is complete."""
@@ -953,7 +1010,7 @@ class Experiment(object):
     def fail_participant(self, participant):
         """Fail all the nodes of a participant."""
         participant_nodes = (
-            self.session.query(Node)
+            db.session.query(Node)
             .filter_by(participant_id=participant.id, failed=False)
             .all()
         )
@@ -1160,7 +1217,7 @@ class Experiment(object):
         order they were created.
         """
         if session is None:
-            session = self.session
+            session = db.session
         return session.query(Info).order_by(Info.creation_time)
 
     def replay_event(self, event):
@@ -1214,9 +1271,9 @@ class Experiment(object):
         :returns: An ``OrderedDict()`` mapping panel titles to data structures
                   describing the experiment state.
         """  # noqa
-        participants = self.session.query(Participant)
-        nodes = self.session.query(Node)
-        infos = self.session.query(Info)
+        participants = db.session.query(Participant)
+        nodes = db.session.query(Node)
+        infos = db.session.query(Info)
 
         unique_statuses = set(participant.status for participant in participants.all())
         stats = OrderedDict()
@@ -1226,7 +1283,7 @@ class Experiment(object):
         )
 
         # Count up our networks by role
-        network_roles = self.session.query(Network.role, func.count(Network.role))
+        network_roles = db.session.query(Network.role, func.count(Network.role))
         network_counts = network_roles.group_by(Network.role).all()
         failed_networks = network_roles.filter(Network.failed == True)  # noqa
         failed_counts = dict(failed_networks.group_by(Network.role).all())
@@ -1360,7 +1417,7 @@ class Experiment(object):
 
         if polymorphic_identity is None and "type" in table.columns:
             observed_types = [
-                r.type for r in self.session.query(table.columns.type).distinct().all()
+                r.type for r in db.session.query(table.columns.type).distinct().all()
             ]
             obj_by_type = [
                 self.pull_table(
@@ -1424,62 +1481,362 @@ class Experiment(object):
 
         model = getattr(models, object_type, None)
         if model is not None:
-            obj = self.session.query(model).get(int(obj_id))
+            obj = db.session.query(model).get(int(obj_id))
             if getattr(obj, "visualization_html", None):
                 return obj.visualization_html
         return ""
 
+    def resolve_attr(self, cls, key):
+        """
+        Resolve a DataTables column key to a SQLAlchemy column and optional
+        transforms for display and filtering.
+        """
+        attr = None
+
+        def identity(x):
+            return "" if x is None else str(x)
+
+        label_fn = identity
+        filter_fn = identity
+
+        if key == "object_type":
+            maybe_attr = getattr(cls, "type", None)
+            if isinstance(maybe_attr, InstrumentedAttribute):
+                attr = maybe_attr
+
+                def _label_fn(s):
+                    return (s or "").capitalize()
+
+                def _filter_fn(s):
+                    return (s or "").lower()
+
+                label_fn = _label_fn
+                filter_fn = _filter_fn
+        else:
+            maybe_attr = getattr(cls, key, None)
+            if isinstance(maybe_attr, InstrumentedAttribute):
+                attr = maybe_attr
+        return attr, label_fn, filter_fn
+
     def table_data(
-        self, table: str = "participant", polymorphic_identity: Optional[str] = None
+        self,
+        start: int,
+        length: int,
+        table: str = "participant",
+        polymorphic_identity: Optional[str] = None,
+        search_value: str = "",
+        order_column: Optional[str] = None,
+        order_dir: str = "asc",
+        column_filters: Optional[dict[str, list[str]]] = None,
     ):
-        """Generates DataTablesJS data and configuration for the experiment. The data
-        is compiled from the models' ``__json__`` methods, and can be customized by either
-        overriding this method or using the ``json_data`` method on the model to return
-        additional serializable data.
+        """
+        Generates server-side paginated DataTablesJS data for the experiment.
 
-        :param table: table to query
+        Rows are queried directly from the database using SQLAlchemy, filtered,
+        searched, ordered, and paginated according to DataTables' request
+        parameters. The data is compiled from the models' ``__json__`` methods,
+        and may be customized by overriding this method or by having models
+        return additional serializable data in their ``__json__``.
 
-        :param polymorphic_identity: optional polymorphic identity (corresponds to the ``type`` column)
-
-        :returns: Returns a ``dict`` with DataTablesJS data and configuration, filters using
-                  arbitrary keyword arguments. Should contain ``data`` and ``columns`` keys
-                  at least, with ``columns`` containing data for all fields on all returned
-                  objects.
-        """  # noqa
-        rows = []
-        found_columns = set()
-        columns = []
-
-        table = Base.metadata.tables[table]
+        :param start: Starting record index (0-based), provided by DataTables.
+        :param length: Number of records to return, provided by DataTables.
+        :param table: Name of the table to query (default: "participant").
+        :param polymorphic_identity: Optional polymorphic identity, corresponding
+            to the ``type`` column, used to restrict results to a subclass.
+        :param search_value: Global search string to filter results (default: "").
+        :param order_column: Column name to sort by (default: None = primary key).
+        :param order_dir: Sort direction, "asc" or "desc" (default: "asc").
 
+        :returns: A ``dict`` with keys:
+            - ``data``: List of row dicts for the current page.
+            - ``total_count``: Total number of rows before filtering.
+            - ``filtered_count``: Number of rows after filtering.
+        """
+        table_obj = Base.metadata.tables[table]
         if polymorphic_identity == "None":
             polymorphic_identity = None
 
-        objects = self.pull_table(table, polymorphic_identity=polymorphic_identity)
+        if polymorphic_identity is None:
+            cls = get_mapped_class(table_obj)
+            base = self.session.query(cls)
+        else:
+            cls = get_polymorphic_mapping(table_obj)[polymorphic_identity]
+            base = self.session.query(cls).filter(cls.type == polymorphic_identity)
 
-        for obj in objects:
-            data = obj.__json__()
-            # Add participant worker_id to data, we normally leave it out of
-            # JSON renderings
-            if table.name == "participant":
+        total_count = base.order_by(None).count()
+
+        # Global search
+        q = base
+        if search_value:
+            conds = []
+            for col in table_obj.columns:
+                if hasattr(cls, col.name):
+                    conds.append(
+                        cast(getattr(cls, col.name), String).ilike(f"%{search_value}%")
+                    )
+            if conds:
+                q = q.filter(or_(*conds))
+
+        # Apply SearchPanes column selections (exact match on string-cast)
+        column_filters = column_filters or {}
+        for key, selected in column_filters.items():
+            if not selected:
+                continue
+            attr, _, to_db = self.resolve_attr(cls, key)
+            if attr is None:
+                continue
+            q = q.filter(cast(attr, String).in_([to_db(v) for v in selected]))
+
+        filtered_count = q.order_by(None).count()
+
+        # Ordering
+        attr = getattr(cls, order_column, None) if order_column else None
+        if isinstance(attr, InstrumentedAttribute):
+            q = q.order_by(desc(attr) if order_dir.lower() == "desc" else asc(attr))
+        else:
+            # Fallback: order by primary key(s)
+            for pk in table_obj.primary_key.columns:
+                pk_attr = getattr(cls, pk.name, None)
+                if isinstance(pk_attr, InstrumentedAttribute):
+                    q = q.order_by(pk_attr)
+
+        # Page
+        items = q.offset(start).limit(length).all()
+
+        # Rows (strings escaped; non-strings pretty-printed inside <code>)
+        rows, all_keys = [], set()
+        for obj in items:
+            data = obj.__json__() or {}
+            if table_obj.name == "participant" and hasattr(obj, "worker_id"):
                 data["worker_id"] = obj.worker_id
-            rows.append(data)
-            for key in data:
-                if key not in found_columns:
-                    columns.append({"name": key, "data": key})
-                    found_columns.add(key)
 
-        # Make sure every row has an entry for every column
-        for col in found_columns:
-            for row in rows:
-                if col not in row:
-                    row[col] = None
+            coerced = {}
+            for key, value in data.items():
+                if value is None:
+                    coerced[key] = None
+                elif isinstance(value, (str, bytes)):
+                    coerced[key] = escape(value)
+                else:
+                    coerced[key] = (
+                        f"<code>{escape(json.dumps(value, default=date_handler))}</code>"
+                    )
+            rows.append(coerced)
+            all_keys.update(coerced.keys())
+
+        for row in rows:
+            for key in all_keys:
+                row.setdefault(key, None)
 
         return {
             "data": rows,
-            "columns": columns,
+            "total_count": total_count,
+            "filtered_count": filtered_count,
         }
 
+    def table_search_panes(
+        self,
+        table: str,
+        polymorphic_identity: Optional[str],
+        search_value: str,
+        pane_columns: list[str],
+        column_filters: dict[str, list[str]],
+        threshold: float,
+        max_distinct: int = 200,
+    ):
+        """
+        Compute SearchPanes options for the provided columns using server-side logic.
+
+        Mirrors client behavior:
+        - Applies global search and other panes' selections.
+        - For each column, first checks distinct count vs. threshold:
+        if distinct_count / filtered_rows > threshold or distinct_count > max_distinct,
+        the pane is omitted (empty list).
+        - Otherwise, returns {label, value, total, count} for that column.
+
+        :param table: Table name.
+        :param polymorphic_identity: Optional polymorphic identity ('type' column filter).
+        :param search_value: Global search string.
+        :param pane_columns: Ordered list of column keys (from DataTables).
+        :param column_filters: Current pane selections: { key: [values...] }.
+        :param threshold: Pane display threshold (same as DataTables config).
+        :param max_distinct: Safety cap on distinct values per pane.
+        :returns: ``{"options": { <col_key>: [ {label,value,total,count}, ... ], ... }}``
+        """
+        table_obj = Base.metadata.tables[table]
+        if polymorphic_identity == "None":
+            polymorphic_identity = None
+
+        if polymorphic_identity is None:
+            cls = get_mapped_class(table_obj)
+            base = self.session.query(cls)
+        else:
+            cls = get_polymorphic_mapping(table_obj)[polymorphic_identity]
+            base = self.session.query(cls).filter(cls.type == polymorphic_identity)
+
+        # Build q_global: global search ONLY (no panes)
+        def apply_global_search(q):
+            if not search_value:
+                return q
+            conditions = [
+                cast(getattr(cls, c.name), String).ilike(f"%{search_value}%")
+                for c in table_obj.columns
+                if hasattr(cls, c.name)
+            ]
+            return q.filter(or_(*conditions)) if conditions else q
+
+        q_global = apply_global_search(base).order_by(None)
+        global_count = q_global.count()
+
+        # Build q_all: global search + ALL panes filters
+        q_all = q_global
+        for key, selected in (column_filters or {}).items():
+            if not selected:
+                continue
+            attr, _, to_db = self.resolve_attr(cls, key)
+            if attr is not None:
+                q_all = q_all.filter(
+                    cast(attr, String).in_([to_db(v) for v in selected])
+                )
+
+        panes_options: dict[str, list[dict]] = {}
+
+        for key in pane_columns:
+            # Resolve the column attribute and label/value transforms
+            attr, to_label, to_db = self.resolve_attr(cls, key)
+            if attr is None:
+                panes_options[key] = []
+                continue
+            attr_str = cast(attr, String)
+
+            # Always show if this pane currently has a selection
+            has_selection = bool((column_filters or {}).get(key))
+
+            # Eligibility: distincts under q_global (NOT q_all)
+            elig_vals = (
+                q_global.with_entities(attr_str.label("v"))
+                .group_by(attr_str)
+                .order_by(func.count().desc())
+                .limit(max_distinct + 1)
+                .all()
+            )
+            distinct_count = len(elig_vals)
+            ratio = (distinct_count / global_count) if global_count else 0.0
+
+            if not has_selection and (
+                distinct_count > max_distinct or ratio > threshold
+            ):
+                # Hide this pane (too many uniques)
+                panes_options[key] = []
+                continue
+
+            # Totals: q_global grouped
+            totals = (
+                q_global.with_entities(attr_str.label("val"), func.count().label("cnt"))
+                .group_by(attr_str)
+                .order_by(func.count().desc())
+                .limit(max_distinct)
+                .all()
+            )
+            totals_map = {("" if v is None else str(v)): int(cnt) for v, cnt in totals}
+
+            # Counts: q_all grouped
+            counts = (
+                q_all.with_entities(attr_str.label("val"), func.count().label("cnt"))
+                .group_by(attr_str)
+                .order_by(func.count().desc())
+                .limit(max_distinct)
+                .all()
+            )
+            counts_map = {("" if v is None else str(v)): int(cnt) for v, cnt in counts}
+
+            # Union (so badges show for all values)
+            values = set(totals_map) | set(counts_map)
+            col_opts: list[dict] = []
+            for raw in values:
+                label = to_label(raw)
+                value = label if key == "object_type" else raw
+                col_opts.append(
+                    {
+                        "label": label,
+                        "value": value,
+                        "total": totals_map.get(raw, 0),
+                        "count": counts_map.get(raw, 0),
+                    }
+                )
+            panes_options[key] = col_opts
+
+        return {"options": panes_options}
+
+    def table_columns(
+        self,
+        table: str = "participant",
+        polymorphic_identity: Optional[str] = None,
+    ) -> list[dict]:
+        """
+        Return column definitions for the header in DB schema order.
+        A single aggregate query is used to keep only columns that have at least
+        one non-empty value (NULL and '' are treated as empty).
+        For the 'participant' table, 'worker_id' is always appended since it is
+        injected into row data in `table_data()`.
+
+        :param table: Table name to inspect (default: "participant").
+        :param polymorphic_identity: Optional subtype filter (uses 'type' column).
+        :returns: List of { "name": <col>, "data": <col> } in schema order, plus worker_id for participants.
+        """
+        table_obj = Base.metadata.tables[table]
+
+        if polymorphic_identity in (None, "None"):
+            cls = get_mapped_class(table_obj)
+            q = self.session.query(cls)
+        else:
+            cls = get_polymorphic_mapping(table_obj)[polymorphic_identity]
+            q = self.session.query(cls).filter(cls.type == polymorphic_identity)
+
+        exprs, names = [], []
+        for column in table_obj.columns:
+            attr = getattr(cls, column.name, None)
+            if attr is None:
+                continue
+            # COUNT(NULLIF(CAST(col AS TEXT), '')) counts only non-empty, non-null
+            exprs.append(
+                func.count(func.nullif(cast(attr, String), "")).label(column.name)
+            )
+            names.append(column.name)
+
+        nonempty_counts = []
+        if exprs:
+            nonempty_counts = list(self.session.query(*exprs).one())
+
+        # Fetch one object to obtain its JSON representation and filter out unneeded columns
+        obj = q.order_by(None).limit(1).first()
+        json_columns: set[str] = set()
+        if obj is not None:
+            data = obj.__json__() or {}
+            if isinstance(data, dict):
+                json_columns.update(data.keys())
+
+        # Keep columns with count > 0, preserving schema order, but only those present in __json__()
+        cols: list[dict] = []
+        for name, count in zip(names, nonempty_counts):
+            if count and int(count) > 0 and name in json_columns:
+                cols.append({"name": name, "data": name})
+
+        if table_obj.name == "participant":
+            if all(c["data"] != "worker_id" for c in cols):
+                cols.append({"name": "worker_id", "data": "worker_id"})
+
+        # Fallback for empty tables
+        if not cols:
+            cols = [
+                {"name": column.name, "data": column.name}
+                for column in table_obj.columns
+            ]
+            if table_obj.name == "participant":
+                if all(c["data"] != "worker_id" for c in cols):
+                    cols.append({"name": "worker_id", "data": "worker_id"})
+
+        return cols
+
     def dashboard_database_actions(self):
         """Returns a sequence of custom actions for the database dashboard. Each action
         must have a ``title`` and a ``name`` corresponding to a method on the
@@ -1513,7 +1870,7 @@ class Experiment(object):
             object_type = entry.get("object_type")
             model = getattr(models, object_type, None)
             if model is not None:
-                obj = self.session.query(model).get(int(obj_id))
+                obj = db.session.query(model).get(int(obj_id))
                 if obj is not None and not obj.failed:
                     obj.fail()
                     counts[object_type] = counts.get(object_type, 0) + 1
@@ -1532,7 +1889,7 @@ class Experiment(object):
 
     @contextmanager
     def restore_state_from_replay(
-        self, app_id, session, zip_path=None, **configuration_options
+        self, app_id, session=None, zip_path=None, **configuration_options
     ):
         # We need to fake dallinger_experiment to point at the current experiment
         module = sys.modules[type(self).__module__]
@@ -1540,23 +1897,10 @@ class Experiment(object):
             logger.warning("dallinger_experiment is already set, updating")
         sys.modules["dallinger_experiment"] = module
 
-        # Load the configuration system and globals
-        config = get_config()
-        # Manually load extra parameters and ignore errors
-        try:
-            from dallinger_experiment.experiment import extra_parameters
+        # Load the configuration
+        config = get_config(load=True)
 
-            try:
-                extra_parameters()
-                config._module_params_loaded = True
-            except KeyError:
-                pass
-        except ImportError:
-            pass
-
-        config.load()
         self.app_id = self.original_app_id = app_id
-        self.session = session
         self.exp_config = config
 
         # The replay index is initialised to 1970 as that is guaranteed
@@ -1612,8 +1956,8 @@ class Experiment(object):
         # Clear up global state
         self.import_session.rollback()
         self.import_session.close()
-        session.rollback()
-        session.close()
+        db.session.rollback()
+        db.session.close()
         config._reset(register_defaults=True)
         del sys.modules["dallinger_experiment"]
 
