@@ -1,9 +1,9 @@
 """A home for all Dallinger Experiments. Experiments should be registered
 with a ``setuptools`` ``entry_point`` for the ``dallinger.experiments`` group.
 """
-import logging
 
-from pkg_resources import iter_entry_points
+import logging
+from importlib.metadata import entry_points
 
 from ..experiment import Experiment
 
@@ -13,7 +13,9 @@ logger.addHandler(logging.NullHandler())
 # Avoid PEP-8 warning, we want this to be importable from this location
 Experiment = Experiment
 
-for entry_point in iter_entry_points(group="dallinger.experiments"):
+experiments_entry_points = entry_points(group="dallinger.experiments")
+
+for entry_point in experiments_entry_points:
     try:
         globals()[entry_point.name] = entry_point.load()
     except ImportError:
