@@ -14,11 +14,12 @@ from sqlalchemy import (
     String,
     Text,
     and_,
+    func,
     or_,
 )
 from sqlalchemy.dialects.postgresql import JSONB
-from sqlalchemy.orm import relationship, validates
-from sqlalchemy.sql.expression import false
+from sqlalchemy.orm import column_property, relationship, validates
+from sqlalchemy.sql.expression import false, select
 
 from .db import Base
 
@@ -195,7 +196,7 @@ class Participant(Base, SharedMixin):
 
     #: a String giving the name of the class. Defaults to
     #: "participant". This allows subclassing.
-    type = Column(String(50))
+    type = Column(String)
     __mapper_args__ = {"polymorphic_on": type, "polymorphic_identity": "participant"}
 
     #: A String, the fingerprint hash of the participant.
@@ -212,7 +213,7 @@ class Participant(Base, SharedMixin):
 
     #: A String, a concatenation of :attr:`~dallinger.models.Participant.worker_id`
     #: and :attr:`~dallinger.models.Participant.assignment_id`
-    unique_id = Column(String(75), nullable=False, index=True)
+    unique_id = Column(String(150), nullable=False, index=True)
 
     #: A String, the id of the hit the participant is working on
     hit_id = Column(String(50), nullable=False)
@@ -238,34 +239,39 @@ class Participant(Base, SharedMixin):
 
     #: String representing the current status of the participant, can be:
     #:
-    #:    - ``working`` - participant is working
-    #:    - ``overrecruited`` - number of recruited participants exceed number required for the experiment, so this participant will not be used
-    #:    - ``submitted`` - participant has submitted their work
-    #:    - ``approved`` - their work has been approved and they have been paid
-    #:    - ``rejected`` - their work has been rejected
-    #:    - ``returned`` - they returned the hit before finishing
     #:    - ``abandoned`` - they ran out of time
-    #:    - ``did_not_attend`` - the participant finished, but failed the
-    #:      attention check
+    #:    - ``approved`` - their work has been approved and they have been paid
     #:    - ``bad_data`` - the participant finished, but their data was
     #:      malformed
+    #:    - ``did_not_attend`` - the participant finished, but failed the
+    #:      attention check
     #:    - ``missing_notification`` - this indicates that Dallinger has
     #:      inferred that a Mechanical Turk notification corresponding to this
     #:      participant failed to arrive. This is an uncommon, but potentially
     #:      serious issue.
+    #:    - ``overrecruited`` - number of recruited participants exceed number required for the experiment, so this participant will not be used
+    #:    - ``recruiter_submission_started`` - participant has submitted their assignment to their recruiter, but the recruiter is still processing it
+    #:    - ``rejected`` - their work has been rejected
+    #:    - ``replaced`` - they have been replaced by another participant
+    #:    - ``returned`` - they returned the hit before finishing
+    #:    - ``screened_out`` - they were screened out (Prolific only)
+    #:    - ``submitted`` - participant's assignment submission processing has been completed by their recruiter
+    #:    - ``working`` - participant is working
     status = Column(
         Enum(
-            "working",
-            "overrecruited",
-            "submitted",
-            "approved",
-            "rejected",
-            "returned",
             "abandoned",
-            "did_not_attend",
+            "approved",
             "bad_data",
+            "did_not_attend",
             "missing_notification",
+            "overrecruited",
+            "recruiter_submission_started",
+            "rejected",
             "replaced",
+            "returned",
+            "screened_out",
+            "submitted",
+            "working",
             name="participant_status",
         ),
         nullable=False,
@@ -389,7 +395,7 @@ class Question(Base, SharedMixin):
 
     #: a String giving the name of the class. Defaults to
     #: "question". This allows subclassing.
-    type = Column(String(50))
+    type = Column(String)
     __mapper_args__ = {"polymorphic_on": type, "polymorphic_identity": "question"}
 
     #: the participant who made the response
@@ -443,7 +449,7 @@ class Network(Base, SharedMixin):
 
     #: A String giving the name of the class. Defaults to
     #: "network". This allows subclassing.
-    type = Column(String(50))
+    type = Column(String)
 
     __mapper_args__ = {"polymorphic_on": type, "polymorphic_identity": "network"}
 
@@ -474,12 +480,17 @@ class Network(Base, SharedMixin):
         )
 
     def json_data(self):
-        """Return json description of a participant."""
+        """Return json description of a network."""
         return {
             "type": self.type,
             "max_size": self.max_size,
             "full": self.full,
             "role": self.role,
+            "n_pending_infos": self.n_pending_infos,
+            "n_completed_infos": self.n_completed_infos,
+            "n_failed_infos": self.n_failed_infos,
+            "n_alive_nodes": self.n_alive_nodes,
+            "n_failed_nodes": self.n_failed_nodes,
             "object_type": "Network",
         }
 
@@ -675,7 +686,7 @@ class Node(Base, SharedMixin):
 
     #: A String giving the name of the class. Defaults to
     #: ``node``. This allows subclassing.
-    type = Column(String(50))
+    type = Column(String)
     __mapper_args__ = {"polymorphic_on": type, "polymorphic_identity": "node"}
 
     #: the id of the network that this node is a part of
@@ -1393,7 +1404,7 @@ class Vector(Base, SharedMixin):
     #: Note: The type column was added in 9/2022, 7+ years after the Vector ORM class was introduced. To support
     #: importing datasets which don't include this column we define a default value which will be used when deploying
     #: experiments from zip files.
-    type = Column(String(50), default="vector", server_default="vector")
+    type = Column(String, default="vector", server_default="vector")
     __mapper_args__ = {"polymorphic_on": type, "polymorphic_identity": "vector"}
 
     #: the id of the Node at which the vector originates
@@ -1514,7 +1525,7 @@ class Info(Base, SharedMixin):
 
     #: a String giving the name of the class. Defaults to "info".
     #: This allows subclassing.
-    type = Column(String(50))
+    type = Column(String)
     __mapper_args__ = {"polymorphic_on": type, "polymorphic_identity": "info"}
 
     #: the id of the Node that created the info
@@ -1532,6 +1543,9 @@ class Info(Base, SharedMixin):
     #: the contents of the info. Must be stored as a String.
     contents = Column(Text(), default=None)
 
+    #: whether the info is 'complete', i.e. has received its contents
+    complete = Column(Boolean(), default=False)
+
     def __init__(self, origin, contents=None, details=None, failed=False):
         """Create an info."""
         # check the origin hasn't failed
@@ -1557,6 +1571,15 @@ class Info(Base, SharedMixin):
             raise ValueError("The contents of an info is write-once.")
         return value
 
+    def __setattr__(self, key, value):
+        super().__setattr__(key, value)
+        if key != "complete":
+            self.check_complete()
+
+    def check_complete(self):
+        if self.contents is not None:
+            self.complete = True
+
     def __repr__(self):
         """The string representation of an info."""
         return "Info-{}-{}".format(self.id, self.type)
@@ -1760,7 +1783,7 @@ class Transformation(Base, SharedMixin):
 
     #: a String giving the name of the class. Defaults to
     #: "transformation". This allows subclassing.
-    type = Column(String(50))
+    type = Column(String)
     __mapper_args__ = {"polymorphic_on": type, "polymorphic_identity": "transformation"}
 
     #: the id of the info that was transformed.
@@ -1849,6 +1872,13 @@ class Notification(Base, SharedMixin):
     # the type of notification
     event_type = Column(String, nullable=False)
 
+    def json_data(self):
+        """The json representation of a notification."""
+        return {
+            "assignment_id": self.assignment_id,
+            "event_type": self.event_type,
+        }
+
 
 class Recruitment(Base, SharedMixin):
     """A record of a request to recruit a participant."""
@@ -1857,3 +1887,51 @@ class Recruitment(Base, SharedMixin):
 
     #: A String, the nickname of the recruiter used.
     recruiter_id = Column(String(50), nullable=True)
+
+
+Network.n_pending_infos = column_property(
+    select(func.count(Info.id))
+    .where(
+        Info.network_id == Network.id,
+        ~Info.failed,
+        ~Info.complete,
+    )
+    .scalar_subquery()
+)
+
+Network.n_completed_infos = column_property(
+    select(func.count(Info.id))
+    .where(
+        Info.network_id == Network.id,
+        ~Info.failed,
+        Info.complete,
+    )
+    .scalar_subquery()
+)
+
+Network.n_failed_infos = column_property(
+    select(func.count(Info.id))
+    .where(
+        Info.network_id == Network.id,
+        Info.failed,
+    )
+    .scalar_subquery()
+)
+
+Network.n_alive_nodes = column_property(
+    select(func.count(Node.id))
+    .where(
+        Node.network_id == Network.id,
+        ~Node.failed,
+    )
+    .scalar_subquery()
+)
+
+Network.n_failed_nodes = column_property(
+    select(func.count(Node.id))
+    .where(
+        Node.network_id == Network.id,
+        Node.failed,
+    )
+    .scalar_subquery()
+)
