@@ -104,13 +104,15 @@ class ProlificService:
         the study on Prolific. If we get there first, there will be an error
         because the submission hasn't happened yet.
         """
-        from dallinger.recruiters import handle_and_raise_recruitment_error
-
         status = self.get_participant_submission(submission_id)["status"]
-        if status != "AWAITING REVIEW":
+        if status == "APPROVED":
+            logger.info(
+                "Participant submission is already approved, no need to approve again."
+            )
+        elif status != "AWAITING REVIEW":
             # This will trigger a retry from the decorator
-            handle_and_raise_recruitment_error(
-                ProlificServiceException("Prolific session not yet submitted.")
+            raise ProlificServiceException(
+                f"Prolific session not yet submitted (current status is '{status}')."
             )
 
         return self._req(
@@ -608,8 +610,7 @@ class DevProlificService(ProlificService):
 
         from dallinger.config import get_config
 
-        config = get_config()
-        config.load()
+        config = get_config(load=True)
 
         workspace = config.get("prolific_workspace")
         is_workspace_id = (
