@@ -494,9 +494,7 @@ class ProlificRecruiter(Recruiter):
 
     def __init__(self, *args, **kwargs):
         super().__init__()
-        self.config = get_config()
-        if not self.config.ready:
-            self.config.load()
+        self.config = get_config(load=True)
         base_url = get_base_url()
         self.ad_url = f"{base_url}/ad?recruiter={self.nickname}"
         self.study_domain = os.getenv("HOST")
@@ -650,6 +648,10 @@ class ProlificRecruiter(Recruiter):
                 submission_id=assignment_id
             )
         except ProlificServiceException as ex:
+            logger.warning(
+                f"approve_participant_submission for assignment_id '{assignment_id}' "
+                f"failed with error '{str(ex)}'. Will try to proceed anyway."
+            )
             handle_recruitment_error(ex)
 
     def close_recruitment(self):
@@ -1436,9 +1438,7 @@ class MTurkRecruiter(Recruiter):
 
     def __init__(self, *args, **kwargs):
         super().__init__()
-        self.config = get_config()
-        if not self.config.ready:
-            self.config.load()
+        self.config = get_config(load=True)
         base_url = get_base_url()
         self.ad_url = "{}/ad?recruiter={}".format(base_url, self.nickname)
         self.notification_url = "{}/mturk-sns-listener".format(base_url)
@@ -1546,15 +1546,22 @@ class MTurkRecruiter(Recruiter):
 
     def compensate_worker(self, worker_id, email, dollars, notify=True):
         """Pay a worker by means of a special HIT that only they can see."""
-        qualification = self.mturkservice.create_qualification_type(
-            name="Dallinger Compensation Qualification - {}".format(
-                generate_random_id()
-            ),
-            description=(
-                "You have received a qualification to allow you to complete a "
-                "compensation HIT from Dallinger for ${}.".format(dollars)
-            ),
-        )
+        try:
+            qualification = self.mturkservice.create_qualification_type(
+                name="Dallinger Compensation Qualification - {}".format(
+                    generate_random_id()
+                ),
+                description=(
+                    "You have received a qualification to allow you to complete a "
+                    "compensation HIT from Dallinger for ${}.".format(dollars)
+                ),
+            )
+        except Exception as ex:
+            handle_and_raise_recruitment_error(
+                MTurkServiceException(
+                    "Failed to create compensation qualification: {}".format(str(ex))
+                )
+            )
         qid = qualification["id"]
         self.mturkservice.assign_qualification(qid, worker_id, 1, notify=notify)
         hit_request = {
@@ -1774,12 +1781,7 @@ class MTurkRecruiter(Recruiter):
             mturk_type = event.get("EventType")
             assignment_id = event.get("AssignmentId")
 
-            if mturk_type in [
-                "AssignmentAbandoned",
-                "AssignmentAccepted",
-                "AssignmentReturned",
-                "AssignmentSubmitted",
-            ]:
+            if assignment_id:
                 participant = (
                     session.query(Participant)
                     .filter_by(assignment_id=assignment_id)
@@ -1787,22 +1789,38 @@ class MTurkRecruiter(Recruiter):
                     .first()
                 )
                 if participant is None:
-                    logger.error(
+                    msg = (
                         f"Received an {mturk_type} notification from MTurk for assignment ID {assignment_id}, "
                         "which is not related to any participant."
                     )
-                    return
+                    if mturk_type == "AssignmentSubmitted":
+                        # For submitted assignments we expect a corresponding Participant to exist.
+                        raise MTurkRecruiterException(
+                            msg
+                            + " This is invalid for AssignmentSubmitted; investigate participant creation and notification routing."
+                        )
+                    logger.error(msg)
+                    continue
 
-                if mturk_type == "AssignmentAbandoned":
-                    participant.status = "abandoned"
-                if mturk_type == "AssignmentReturned":
-                    participant.status = "returned"
-                if mturk_type == "AssignmentSubmitted":
-                    participant.status = "submitted"
-                session.commit()
+                # Only handle and enqueue for supported event types
+                if mturk_type in [
+                    "AssignmentAbandoned",
+                    "AssignmentAccepted",
+                    "AssignmentReturned",
+                    "AssignmentSubmitted",
+                ]:
+                    if mturk_type == "AssignmentAbandoned":
+                        participant.status = "abandoned"
+                    if mturk_type == "AssignmentReturned":
+                        participant.status = "returned"
+                    if mturk_type == "AssignmentSubmitted":
+                        participant.status = "submitted"
+                    session.commit()
 
-            dlgr_event_type = self._translate_event_type(mturk_type)
-            q.enqueue(worker_function, dlgr_event_type, assignment_id, participant.id)
+                    dlgr_event_type = self._translate_event_type(mturk_type)
+                    q.enqueue(
+                        worker_function, dlgr_event_type, assignment_id, participant.id
+                    )
 
     def _mturk_status_for(self, participant):
         try:
@@ -1861,6 +1879,12 @@ class MTurkRecruiter(Recruiter):
                 )
             except DuplicateQualificationNameError:
                 result["existing_qualifications"].append(name)
+            except Exception as ex:
+                handle_and_raise_recruitment_error(
+                    MTurkServiceException(
+                        "Failed to create qualification: {}".format(str(ex))
+                    )
+                )
 
         # We need to make sure the new qualifications are actually ready
         # for assignment, as there's a small delay.
