@@ -96,23 +96,29 @@ class TestVerify(object):
             "dallinger.command_line.utils.ExperimentFileSource.size",
             new_callable=mock.PropertyMock,
         ) as size:
-            size.return_value = 6000000  # 6 MB, so over the limit
-            assert v_directory(max_size_mb=5) is False
+            size.return_value = 300000000  # 300 MB, so over the limit
+            with mock.patch.dict(os.environ, {"EXP_MAX_SIZE_MB": "256"}):
+                assert v_directory() is False
 
     def test_under_limit_returns_true(self, v_directory):
         with mock.patch(
             "dallinger.command_line.utils.ExperimentFileSource.size",
             new_callable=mock.PropertyMock,
         ) as size:
-            size.return_value = 4000000  # 4 MB, so under the limit
-            assert v_directory(max_size_mb=5) is True
+            size.return_value = 200000000  # 200 MB, so under the limit
+            with mock.patch.dict(os.environ, {"EXP_MAX_SIZE_MB": "256"}):
+                assert v_directory() is True
 
 
 @pytest.mark.slow
 class TestCommandLine(object):
     def test_dallinger_no_args(self):
         result = subprocess.run(["dallinger"], capture_output=True, text=True)
-        assert "Usage: dallinger [OPTIONS] COMMAND [ARGS]" in result.stderr
+        # Output may go different places depending on context
+        # (OS version, Github Actions, etc), so merge them for
+        # better predictability
+        output = result.stdout + result.stderr
+        assert "Usage: dallinger [OPTIONS] COMMAND [ARGS]" in output
 
     def test_log_empty(self):
         id = "dlgr-3b9c2aeb"
@@ -252,6 +258,20 @@ class TestDebugCommand(object):
         CliRunner().invoke(debug, [])
         deployment.assert_called_once()
 
+    def test_wrong_python_version_fails(self, debug, deployment):
+        with mock.patch("platform.python_version") as mock_python_version:
+            mock_python_version.return_value = "3.10.0"
+
+            with mock.patch.dict(os.environ, {}, clear=False):
+                os.environ.pop("SKIP_PYTHON_VERSION_CHECK", None)
+                result = CliRunner().invoke(debug, ["--verbose"])
+
+        deployment.assert_not_called()
+        assert (
+            "Running Python version 3.10.0 which is not consistent with the version stated in .python-version"
+            in result.output
+        )
+
 
 @pytest.mark.usefixtures("bartlett_dir", "active_config", "reset_sys_modules")
 class TestSandboxAndDeploy(object):
