@@ -783,13 +783,13 @@ class Testhandle_launch_data(object):
         log = mock.Mock()
 
         try:
-            handler("https://httpbin.org/status/500", log, attempts=1)
+            handler("https://httpbingo.org/status/500", log, attempts=1)
         except requests.exceptions.HTTPError:
             pass
         log.assert_has_calls(
             [
                 mock.call(
-                    "Error parsing response from https://httpbin.org/status/500, check server logs for details.\n"
+                    "Error parsing response from https://httpbingo.org/status/500, check server logs for details.\n"
                 ),
                 mock.call("Experiment launch failed after multiple attempts."),
             ]
@@ -1180,36 +1180,3 @@ class TestLoad(object):
                 mock.call("Local Heroku process terminated."),
             ]
         )
-
-
-class TestConstraints(object):
-    @pytest.mark.slow
-    def test_constraints_generation(self):
-        from dallinger.utils import ensure_constraints_file_presence
-
-        tmp_path = tempfile.mkdtemp()
-        # We will be looking for
-        # https://raw.githubusercontent.com/Dallinger/Dallinger/v[__version__]
-        # so use an older version we know will exist, rather than the current
-        # version, which may not be tagged/released yet:
-
-        # Change this to the current version after release
-        extant_github_tag = "b98f719c1ce851353f7cfcc78362cfaace51bb8d"
-        (Path(tmp_path) / "requirements.txt").write_text("black")
-        with mock.patch("dallinger.utils.__version__", extant_github_tag):
-            ensure_constraints_file_presence(tmp_path)
-            constraints_file = Path(tmp_path) / "constraints.txt"
-            # If not present a `constraints.txt` file will be generated
-            assert constraints_file.exists()
-            if sys.version_info >= (3, 11):
-                assert "toml" not in constraints_file.read_text()
-            else:
-                assert "toml" in constraints_file.read_text()
-
-            # An existing file will be left untouched
-            constraints_file.write_text("foobar")
-            with pytest.raises(ValueError):
-                ensure_constraints_file_presence(tmp_path)
-            assert constraints_file.read_text() == "foobar"
-
-        shutil.rmtree(tmp_path)
