@@ -1,4 +1,5 @@
-import mock
+from unittest import mock
+
 import pytest
 
 try:
@@ -10,10 +11,10 @@ except ImportError:
 @pytest.mark.skipif(ipywidgets is None, reason="ipywidgets is not installed")
 class TestExperimentWidget(object):
     @pytest.fixture
-    def exp(self):
+    def exp(db_session):
         from dallinger.experiment import Experiment
 
-        return Experiment()
+        return Experiment(db_session)
 
     def test_experiment_initializes_widget(self, exp):
         assert exp.widget is not None
@@ -26,7 +27,7 @@ class TestExperimentWidget(object):
     def test_experiment_displays_widget(self, exp):
         with mock.patch("IPython.display.display") as display:
             exp._ipython_display_()
-            assert display.called_once_with(exp.widget)
+            display.assert_called_once_with(exp.widget)
 
     def test_widget_children_no_config(self, exp):
         assert exp.widget.children[1].children[0].value == "Not loaded."
